/* Copyright (C) 2000-1 drscholl@users.sourceforge.net
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details.

   $Id: mkpass.c,v 1.2 2003/01/11 00:43:31 leodav Exp $ */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#ifndef WIN32
#include <unistd.h>
#else
#include "win32-support.h"
#include <direct.h>
#include <io.h>
#endif
#include <stdlib.h>
#include "opennap.h"
#include "debug.h"

/* needed for the random number generation */
time_t Current_Time = 0;

/* needed for win32 standalone build */
#ifdef WIN32
global_t global;

char	Buf[2048];
int	Index_Path_Depth = 2;
int	Log_Level = 0;
u_int	Server_Flags = 0;

int send_to_channel (char *name, char *buf, int buflen) {
    return 0;
}

#endif


static void
usage (void)
{
    fputs("usage: mkpass [-m|-n filename|-c hashedpw] [-u username] -p password\n", stderr);
    fputs(" -n filename  create New userdb 'filename' with given -u and -p\n", stderr);
    fputs(" -c hashedpw  Check hashed password against -p passwd \n", stderr);
    fputs(" -m           Make hashed password from -p passwd\n", stderr);
    fputs(" -u username  use this as Username\n", stderr);
    fputs(" -p password  use this as Password\n", stderr);
    exit(1);
}

int
main (int argc, char **argv)
{
    char *s, *pass = 0, *user = 0, *file = 0, *hpass = 0;
    int i, mode = 0, fd = 0;

    INIT ();
    while ((i = getopt (argc, argv, "u:p:c:n:mvh")) != -1) {
	switch (i) {
	case 'u':
	    user = optarg;
	    break;
	case 'p':
	    pass = optarg;
	    break;
	case 'm':
	    mode = 0;
	    break;
	case 'c':
	    mode = 1;
	    hpass = optarg;
	    break;
	case 'n':
	    mode = 2;
	    file = optarg;
	    break;
	default:
	    usage ();
	}
    }
    switch (mode) {
	case 0:
	    if (!pass) {
	        fputs("Sorry, need a -p password to make a hashed one for you!\n", stderr);
		usage();
	    }
	    Current_Time = time(0);
	    init_random();
	    s = generate_pass( pass );
	    puts(s);
	    if (check_pass(s, pass))
		fputs("Jikes! an error (can't hash this one try with another password)\n", stderr);
	    FREE(s);
	    CLEANUP();
	    break;
	case 1:
	    if (!hpass) {
	        fputs("Sorry, need some hashed password to check!\n", stderr);
		usage();
	    }
	    if (!pass) {
	        fputs("Sorry, need a -p password to check against with\n", stderr);
		usage();
	    }
	    if (check_pass (hpass, pass))
		puts("Nope, invalid password!");
	    else
		puts("OK, they match ;-)");
	    break;
	case 2:
	    if (!pass) {
	        fputs("Sorry, need a -p password to check against with\n", stderr);
		usage();
	    }
	    if (!user) {
	        fputs("Yes, I also need a -u username to do this.\n", stderr);
		usage();
	    }
	    if (!file)
		file = "users";
	    Current_Time = time(0);
	    init_random();
	    s = generate_pass( pass );
	    if (!(fd = open( file, O_CREAT | O_RDWR | O_TRUNC, S_IRUSR | S_IWUSR))) {
	        fprintf(stderr, "Sorry, can't create file %s\n", file);
	    } else {
		fake_fputs( ":version 1\n", fd);
		fake_fputs( user, fd );
		fake_fputs( " ", fd );
		fake_fputs( s, fd );
		fake_fputs( " unknown Elite 0 0 0\n", fd );
	    }
	    close( fd );
	    FREE(s);
	    CLEANUP();
	    puts( "All done!" );
	    break;
    }
    exit (0);
}
